@@ -70,6 +70,7 @@ int w32 = W32;				/* Windows32 environment	*/
 const char *www = "http://www.gnu.org/software/global/";
 int html_count = 0;
 int sep = '/';
+int need_bless;
 const char *save_config;
 const char *save_argv;
 
@@ -78,12 +79,14 @@ char dbpath[MAXPATHLEN];
 char distpath[MAXPATHLEN];
 char gtagsconf[MAXPATHLEN];
 char datadir[MAXPATHLEN];
+char localstatedir[MAXPATHLEN];
 
 char gtags_path[MAXFILLEN];
 char global_path[MAXFILLEN];
 int gtags_exist[GTAGLIM];
 const char *null_device = NULL_DEVICE;
 const char *tmpdir = "/tmp";
+const char *sitekey = "";
 
 /*
  * Order of items in the top page (This should be customisable variable in the future).
@@ -106,7 +109,7 @@ int Fflag;				/* --frame(-F) option		*/
 int gflag;				/* --gtags(-g) option		*/
 int Iflag;				/* --icon(-I) option		*/
 int nflag;				/* --line-number(-n) option	*/
-int Sflag;				/* --secure-cgi(-S) option	*/
+int Sflag;				/* --system-cgi option		*/
 int qflag;
 int vflag;				/* --verbose(-v) option		*/
 int wflag;				/* --warning(-w) option		*/
@@ -131,9 +134,6 @@ int enable_grep = 1;			/* 1: enable grep		*/
 int enable_idutils = 1;			/* 1: enable idutils		*/
 int enable_xhtml = 1;			/* 1: enable XHTML		*/
 
-const char *action_value;
-const char *id_value;
-const char *cgidir;
 const char *main_func = "main";
 const char *cvsweb_url;
 int use_cvs_module;
@@ -244,8 +244,6 @@ const char *normal_suffix = "html";	/* suffix of normal html file	*/
 const char *HTML;
 const char *action = "cgi-bin/global.cgi";/* default action		*/
 const char *completion_action = "cgi-bin/completion.cgi";
-const char *id = "";			/* id (default non)		*/
-int nocgi = 0;				/* 1: don't make cgi-bin/	*/
 int definition_header=NO_HEADER;	/* (NO|BEFORE|RIGHT|AFTER)_HEADER */
 const char *htags_options = NULL;
 const char *include_file_suffixes = "h,hxx,hpp,H,inc.php";
@@ -268,7 +266,7 @@ static struct option const long_options[] = {
         {"line-number", optional_argument, NULL, 'n'},
         {"main-func", required_argument, NULL, 'm'},
         {"other", no_argument, NULL, 'o'},
-        {"secure-cgi", required_argument, NULL, 'S'},
+        {"system-cgi", required_argument, NULL, 'S'},
         {"symbol", no_argument, NULL, 's'},
         {"table-flist", optional_argument, NULL, 'T'},
         {"title", required_argument, NULL, 't'},
@@ -286,7 +284,6 @@ static struct option const long_options[] = {
         {"disable-idutils", no_argument, &enable_idutils, 0},
         {"full-path", no_argument, &full_path, 1},
         {"html", no_argument, &enable_xhtml, 0},
-        {"nocgi", no_argument, &nocgi, 1},
         {"no-map-file", no_argument, &map_file, 0},
         {"overwrite-key", no_argument, &overwrite_key, 1},
         {"show-position", no_argument, &show_position, 1},
@@ -298,21 +295,18 @@ static struct option const long_options[] = {
         {"help", no_argument, &show_help, 1},
 
 	/* accept value */
-#define OPT_ACTION		128
-#define OPT_CVSWEB		129
-#define OPT_CVSWEB_CVSROOT	130
-#define OPT_GTAGSCONF		131
-#define OPT_GTAGSLABEL		132
-#define OPT_NCOL		133
-#define OPT_ID			134
-#define OPT_INSERT_FOOTER	135
-#define OPT_INSERT_HEADER	136
-#define OPT_ITEM_ORDER		137
-#define OPT_TABS		138
-#define OPT_CFLOW		139
-#define OPT_AUTO_COMPLETION	140
-#define OPT_TREE_VIEW		141
-        {"action", required_argument, NULL, OPT_ACTION},
+#define OPT_CVSWEB		128
+#define OPT_CVSWEB_CVSROOT	129
+#define OPT_GTAGSCONF		130
+#define OPT_GTAGSLABEL		131
+#define OPT_NCOL		132
+#define OPT_INSERT_FOOTER	133
+#define OPT_INSERT_HEADER	134
+#define OPT_ITEM_ORDER		135
+#define OPT_TABS		136
+#define OPT_CFLOW		137
+#define OPT_AUTO_COMPLETION	138
+#define OPT_TREE_VIEW		139
         {"auto-completion", optional_argument, NULL, OPT_AUTO_COMPLETION},
         {"cflow", required_argument, NULL, OPT_CFLOW},
         {"cvsweb", required_argument, NULL, OPT_CVSWEB},
@@ -320,7 +314,6 @@ static struct option const long_options[] = {
         {"gtagsconf", required_argument, NULL, OPT_GTAGSCONF},
         {"gtagslabel", required_argument, NULL, OPT_GTAGSLABEL},
         {"ncol", required_argument, NULL, OPT_NCOL},
-        {"id", required_argument, NULL, OPT_ID},
         {"insert-footer", required_argument, NULL, OPT_INSERT_FOOTER},
         {"insert-header", required_argument, NULL, OPT_INSERT_HEADER},
         {"item-order", required_argument, NULL, OPT_ITEM_ORDER},
@@ -417,7 +410,7 @@ make_file_in_distpath(const char *name, const char *data)
 
 	op = fopen(path, "w");
 	if (op) {
-		if (data) {
+		if (data && *data) {
 			fputs(data, op);
 			fputc('\n', op);
 		}
@@ -460,11 +453,12 @@ load_with_replace(const char *file, STRBUF *result, int place)
 		{"@br@", br},
 		{"@HTML@", HTML},
 		{"@DATADIR@", datadir},
+		{"@LOCALSTATEDIR@", localstatedir},
 		{"@action@", action},
 		{"@completion_action@", completion_action},
 		{"@limit@", auto_completion_limit},
+		{"@sitekey@", sitekey},
 		{"@script_alias@", script_alias},
-		{"@id@", id},
 		{"@null_device@", null_device},
 		{"@globalpath@", global_path},
 		{"@gtagspath@", gtags_path},
@@ -702,7 +696,7 @@ makesearchpart(const char *target)
 	}
 	strbuf_puts_nl(sb, gen_form_begin(target));
 	strbuf_puts_nl(sb, gen_input("pattern", NULL, NULL));
-	strbuf_puts_nl(sb, gen_input("id", id, "hidden"));
+	strbuf_puts_nl(sb, gen_input("id", sitekey, "hidden"));
 	strbuf_puts_nl(sb, gen_input(NULL, "Search", "submit"));
 	strbuf_puts(sb, gen_input(NULL, "Reset", "reset"));
 	strbuf_puts_nl(sb, br);
@@ -843,7 +837,7 @@ makesearchindex(const char *file)
  * makehtaccess: make .htaccess skeleton file.
  */
 static void
-makehtaccess(const char *file)
+makehtaccess(const char *cgidir, const char *file)
 {
 	FILE *op;
 
@@ -862,7 +856,7 @@ makehtaccess(const char *file)
 	fputs_nl("# |AllowOverride Options FileInfo", op);
 	fputs_nl("#", op);
 	fputs_nl("# Htags was invoked with the -f, -c or -D option.", op);
-	fprintf(op, "# You should start HTTP server so that %s/*.cgi is executed\n", cgidir);
+	fprintf(op, "# You should start http server so that %s/*.cgi is executed\n", cgidir);
 	fputs_nl("# as a CGI script.", op);
 	fputs_nl("#", op);
 	fputs_nl("Options +ExecCGI", op);
@@ -871,7 +865,7 @@ makehtaccess(const char *file)
 		fputs_nl("#", op);
 		fputs_nl("# Htags have made gzipped html files because you specified the -c option.", op);
 		fputs_nl("# If your browser doesn't decompress gzipped files, you should start", op);
-		fputs_nl("# HTTP server so that they are decompressed.", op);
+		fputs_nl("# http server so that they are decompressed.", op);
 		fputs_nl("#", op);
 		fputs_nl("# Please rewrite appropriately the string '/cgi-bin/ghtml.cgi' below, or", op);
 		fputs_nl("# copy the file 'cgi-bin/ghtml.cgi' itself to the system's CGI directory.", op);
@@ -1206,6 +1200,10 @@ configuration(int argc, char *const *argv)
 	if (!getconfs("datadir", sb))
 		die("cannot get datadir directory name.");
 	strlimcpy(datadir, strbuf_value(sb), sizeof(datadir));
+	strbuf_reset(sb);
+	if (!getconfs("localstatedir", sb))
+		die("cannot get localstatedir directory name.");
+	strlimcpy(localstatedir, strbuf_value(sb), sizeof(localstatedir));
 	if (getconfn("ncol", &n)) {
 		if (n < 1 || n > 10)
 			warning("parameter 'ncol' ignored because the value (=%d) is too large or too small.", n);
@@ -1516,14 +1514,11 @@ main(int argc, char **argv)
 	if (htags_options)
 		argv = append_options(&argc, argv);
 
-	while ((optchar = getopt_long(argc, argv, "acd:DfFghIm:noqsS:t:Tvwx", long_options, &option_index)) != EOF) {
+	while ((optchar = getopt_long(argc, argv, "acd:DfFghIm:noqst:Tvwx", long_options, &option_index)) != EOF) {
 		switch (optchar) {
 		case 0:
 			/* already flags set */
 			break;
-		case OPT_ACTION:
-			action_value = optarg;
-			break;
 		case OPT_AUTO_COMPLETION:
 			auto_completion = 1;
 			if (optarg) {
@@ -1545,9 +1540,6 @@ main(int argc, char **argv)
 		case OPT_GTAGSCONF:	/* --gtagsconf is estimated only once. */
 		case OPT_GTAGSLABEL:	/* --gtagslabel is estimated only once. */
 			break;
-		case OPT_ID:
-			id_value = optarg;
-			break;
 		case OPT_INSERT_FOOTER:
 			insert_footer = optarg;
 			break;
@@ -1625,7 +1617,7 @@ main(int argc, char **argv)
                         break;
                 case 'S':
 			Sflag++;
-			id = optarg;
+			sitekey = optarg;
                         break;
                 case 'T':
 			table_flist = 1;
@@ -1711,6 +1703,8 @@ main(int argc, char **argv)
                 setquiet();
 		vflag = 0;
 	}
+	if (!cflag && !fflag && !dynamic)
+		Sflag = 0;
 	/*
 	 * If the --xhtml option is specified then all HTML tags which
 	 * are defined in configuration file are ignored. Instead, you can
@@ -1722,7 +1716,9 @@ main(int argc, char **argv)
                 version(av, vflag);
         if (show_help)
                 help();
-
+	/*
+	 * Invokes gtags beforehand.
+	 */
 	if (gflag) {
 		STRBUF *sb = strbuf_open(0);
 
@@ -1751,7 +1747,6 @@ main(int argc, char **argv)
 		strlimcpy(dbpath, arg_dbpath, sizeof(dbpath));
 	else
 		strlimcpy(dbpath, cwdpath, sizeof(dbpath));
-
 	if (cflag && !usable("gzip")) {
 		warning("'gzip' command not found. -c option ignored.");
 		cflag = 0;
@@ -1780,48 +1775,65 @@ main(int argc, char **argv)
 	} else {
 		snprintf(distpath, sizeof(distpath), "%s/HTML", cwdpath);
 	}
+	/*
+	 * Site key management for center CGI.
+	 */
 	if (Sflag) {
+		STRBUF *sb = strbuf_open(0);
 		static char saction[MAXBUFLEN];
 		static char completion_saction[MAXBUFLEN];
 		char path[MAXBUFLEN];
-		char *name = "sitekeys";
-		int fd;
+		int try_writing = 0;
 
 		snprintf(saction, sizeof(saction), "%s/global.cgi", script_alias);
 		action = saction;
 		snprintf(completion_saction, sizeof(completion_saction), "%s/completion.cgi", script_alias);
 		completion_action = completion_saction;
-		snprintf(path, sizeof(path), "%s/gtags", datadir);
-		if (!test("d", makepath(path, name, NULL))) {
+		snprintf(path, sizeof(path), "%s/gtags/%s", localstatedir, SITEKEYDIRNAME);
+		if (!test("d", path)) {
 			setverbose();
-			message("htags: cannot make sitekey file.");
+			message("htags: directory '%s' not found.", path);
 			message("\n[Information]\n");
-			message("Htags was invoked with the -S option. It is required a special site key directory.");
-			message("Please make it by the following command line:");
-			message(" $ mkdir %s/%s", path, name);
-			message(" $ chmod 773 %s/%s", path, name);
-			message("");
-			message("Thank you for your cooperation.");
+			message("It seems that GLOBAL is not install correctly. Please reinstall.");
 			exit(0);
 		}
-		snprintf(path, sizeof(path), "%s/gtags/%s/%s", datadir, name, id);
-		if (test("f", path) && overwrite_key == 0)
-			die("key '%s' is not unique. please change key or use --overwrite-key option.", id);
-		fd = creat(path, 644);
-		if (fd < 0)
-			die("cannot create file '%s'.", path);
-		write(fd, distpath, strlen(distpath));
-		write(fd, "\n", 1);
-		if (fchmod(fd, 0644) < 0)
-			die("cannot chmod file '%s'(errono = %d).", path, errno);
-		close(fd);
-	}
-	/* --action, --id overwrite Sflag's value. */
-	if (action_value) {
-		action = action_value;
-	}
-	if (id_value) {
-		id = id_value;
+		/*
+		 * Load the sitekey if it exists.
+		 */
+		snprintf(path, sizeof(path), "%s/gtags/%s/%s", localstatedir, SITEKEYDIRNAME, sitekey);
+		if (test("f", path)) {
+			FILE *ip = fopen(path, "r");
+
+			if (ip == NULL)
+				die("cannot open file '%s'.", path);
+			strbuf_fgets(sb, ip, STRBUF_NOCRLF);
+			fclose(ip);
+		}
+		if (!test("f", path))
+			try_writing = 1;
+		else if (!strcmp(distpath, strbuf_value(sb)))
+			; /* nothing to do */
+		else if (overwrite_key == 0)
+			die("The site key '%s' is not unique. please change it or use --overwrite-key option.", sitekey);
+		else	/* New key value without --overwrite-key option */
+			try_writing = 1;
+		/*
+		 * In almost case, the following procedures are done
+		 * using bless.sh script by the root user.
+		 */
+		if (try_writing) {
+			FILE *op = fopen(path, "w");
+			
+			if (op == NULL)
+				need_bless = 1;
+			else {
+				fprintf(op, "%s\n", distpath);
+				fclose(op);
+				if (chmod(path, 0644) < 0)
+					die("cannot chmod file '%s'(errno = %d).", path, errno);
+			}
+		}
+		strbuf_close(sb);
 	}
 	/*
 	 * Existence check of tag files.
@@ -1873,17 +1885,6 @@ main(int argc, char **argv)
 	set_env("GTAGSROOT", cwdpath);
 	set_env("GTAGSDBPATH", dbpath);
 	set_env("GTAGSLIBPATH", "");
-	/*
-	 * check directories
-	 */
-	if (fflag || cflag || dynamic) {
-		static char buf[MAXPATHLEN];
-		snprintf(buf, sizeof(buf), "%s/cgi-bin", distpath);
-		cgidir = buf;
-	} else {
-		Sflag = 0;
-		cgidir = NULL;
-	}
 	/*------------------------------------------------------------------
 	 * MAKE FILES
 	 *------------------------------------------------------------------
@@ -1946,14 +1947,14 @@ main(int argc, char **argv)
 	make_directory_in_distpath(SRCS);
 	make_directory_in_distpath(INCS);
 	make_directory_in_distpath(INCREFS);
-	make_file_in_distpath("sitekey", id);
+	make_file_in_distpath("sitekey", sitekey);
 	if (!dynamic) {
 		make_directory_in_distpath(DEFS);
 		make_directory_in_distpath(REFS);
 		if (symbol)
 			make_directory_in_distpath(SYMS);
 	}
-	if (!nocgi && (fflag || cflag || dynamic))
+	if (fflag || cflag || dynamic)
 		make_directory_in_distpath("cgi-bin");
 	if (Iflag)
 		make_directory_in_distpath("icons");
@@ -1962,32 +1963,34 @@ main(int argc, char **argv)
 	/*
 	 * (1) make CGI program
 	 */
-	if (!nocgi && (fflag || cflag || dynamic)) {
+	if (fflag || cflag || dynamic) {
+		char cgidir[MAXPATHLEN];
+		int perm;
+
+		snprintf(cgidir, sizeof(cgidir), "%s/cgi-bin", distpath);
 		message("[%s] (1) making CGI program ...", now());
-		if (cgidir) {
-			/*
-			 * If the Sflag is specified, CGI script is invalidated.
-			 */
-			int perm = Sflag ? 0644 : 0755;
-			if (fflag || dynamic) {
-				makeprogram(cgidir, "global.cgi");
-				if (chmod(makepath(cgidir, "global.cgi", NULL), perm) < 0)
-					die("cannot chmod CGI program.");
-			}
-			if (auto_completion) {
-				makeprogram(cgidir, "completion.cgi");
-				if (chmod(makepath(cgidir, "completion.cgi", NULL), perm) < 0)
-					die("cannot chmod CGI program.");
-			}
-			if (cflag) {
-				makeghtml(cgidir, "ghtml.cgi");
-				if (chmod(makepath(cgidir, "ghtml.cgi", NULL), perm) < 0)
-					die("cannot chmod unzip script.");
-			}
-			makehtaccess(".htaccess");
-			if (chmod(makepath(distpath, ".htaccess", NULL), 0644) < 0)
-				die("cannot chmod .htaccess skeleton.");
+		/*
+		 * If the Sflag is specified, CGI script is invalidated.
+		 */
+		perm = Sflag ? 0644 : 0755;
+		if (fflag || dynamic) {
+			makeprogram(cgidir, "global.cgi");
+			if (chmod(makepath(cgidir, "global.cgi", NULL), perm) < 0)
+				die("cannot chmod CGI program.");
 		}
+		if (auto_completion) {
+			makeprogram(cgidir, "completion.cgi");
+			if (chmod(makepath(cgidir, "completion.cgi", NULL), perm) < 0)
+				die("cannot chmod CGI program.");
+		}
+		if (cflag) {
+			makeghtml(cgidir, "ghtml.cgi");
+			if (chmod(makepath(cgidir, "ghtml.cgi", NULL), perm) < 0)
+				die("cannot chmod unzip script.");
+		}
+		makehtaccess(cgidir, ".htaccess");
+		if (chmod(makepath(distpath, ".htaccess", NULL), 0644) < 0)
+			die("cannot chmod .htaccess skeleton.");
 		/*
 		 * Always make bless.sh.
 		 * Don't grant execute permission to bless script.
@@ -2001,13 +2004,15 @@ main(int argc, char **argv)
 	/*
 	 * Save the suffix of compress format for the safe CGI script.
 	 */
-	if (cflag) {
+	{
 		const char *path = makepath(distpath, "compress", NULL);
 		FILE *op = fopen(path, "w");
 		if (op == NULL)
 			die("cannot make file '%s'.", path);
-		fputs(HTML, op);
-		fputc('\n', op);
+		if (cflag) {
+			fputs(HTML, op);
+			fputc('\n', op);
+		}
 		fclose(op);
 	}
 	if (av) {
@@ -2128,7 +2133,7 @@ main(int argc, char **argv)
 	 */
 	if (enable_xhtml) {
 		char com[MAXPATHLEN*2+32];
-		snprintf(com, sizeof(com), "cp %s/gtags/style.css %s", datadir, distpath);
+		snprintf(com, sizeof(com), "cp %s/gtags/style.css.tmpl %s/style.css", datadir, distpath);
 		system(com);
 	}
 	if (auto_completion || tree_view) {
@@ -2137,16 +2142,25 @@ main(int argc, char **argv)
 		system(com);
 	}
 	message("[%s] Done.", now());
-	if (vflag && !nocgi && (cflag || fflag || dynamic || tree_view)) {
+	if (vflag && (cflag || fflag || dynamic || auto_completion)) {
 		message("\n[Information]\n");
-		if (cflag || fflag || dynamic || tree_view) {
-			message(" o Htags was invoked with the -f, -c, -D or --tree-view option. You should");
-			message("   start a http server so that cgi-bin/*.cgi is executed as a CGI script.");
-		}
+		message(" o Htags was invoked with the -f, -c, -D or --auto-completion option. You should");
+		message("   start http server so that cgi-bin/*.cgi is executed as a CGI script.");
 		if (cflag) {
 			message(" o Htags was invoked with the -c option. You should start a http server to");
 			message("   decompress *.%s files.", gzipped_suffix);
 		}
+		if (Sflag && need_bless) {
+			char path[MAXBUFLEN];
+			snprintf(path, sizeof(path), "%s/gtags/%s/%s", localstatedir, SITEKEYDIRNAME, sitekey);
+
+			message(" o Though htags was invoked with the --system-cgi option with a unique key '%s',", sitekey);
+			message("   you don't have the permission to create file '%s'.", path);
+			message("   You should ask the root user to bless the hypertext by executing");
+			message("   like follows:");
+			message("       # cd HTML");
+			message("       # sh bless.sh");
+		}
  		message("\n If you are using Apache, 'HTML/.htaccess' might be helpful for you.\n");
 		message(" Good luck!\n");
 	}
